@@ -1,20 +1,19 @@
 from __future__ import annotations
 import jax.numpy as np
 from jax import vmap
-from jax.scipy.ndimage import map_coordinates
 from jax.tree_util import tree_map
 from jax.lax import stop_gradient
-from equinox import tree_at, static_field
 from zodiax import ExtendedBase
 from abc import ABC, abstractmethod
 from inspect import signature
+from dLux.utils.helpers import two_image_plot
+from dLux.utils.units import convert_angular, convert_cartesian
 import dLux
 
 
-__all__ = ["CreateWavefront", "TiltWavefront",
-           "NormaliseWavefront", "ApplyBasisOPD", "AddPhase", "AddOPD",
-           "TransmissiveOptic", "CompoundAperture", "ApplyBasisCLIMB",
-           "Rotate"]
+__all__ = ["CreateWavefront", "TiltWavefront", "NormaliseWavefront", 
+           "ApplyBasisOPD", "AddPhase", "AddOPD", "TransmissiveOptic", 
+           "ApplyBasisCLIMB", "Rotate"]
 
 
 Array = np.ndarray
@@ -32,7 +31,7 @@ class OpticalLayer(ExtendedBase, ABC):
     name : str
         The name of the layer, which is used to index the layers dictionary.
     """
-    name : str = static_field()
+    name : str
 
 
     def __init__(self : OpticalLayer,
@@ -103,6 +102,69 @@ class OpticalLayer(ExtendedBase, ABC):
 
         # Return updated parameters dictionary
         return parameters
+    
+
+    def summary(self            : OpticalLayer, 
+                angular_units   : str = 'radians', 
+                cartesian_units : str = 'meters', 
+                sigfigs         : int = 4) -> str:
+        """
+        Returns a summary of the class.
+
+        Parameters
+        ----------
+        angular_units : str = 'radians'
+            The angular units to use in the summary. Options are 'radians', 
+            'degrees', 'arcseconds' and 'arcminutes'.
+        cartesian_units : str = 'meters'
+            The cartesian units to use in the summary. Options are 'meters',
+            'millimeters' and 'microns'.
+        sigfigs : int = 4
+            The number of significant figures to use in the summary.
+
+        Returns
+        -------
+        summary : str
+            A summary of the class.
+        """
+        return f"{self.name} layer has no summary method yet."
+
+
+    def display(self            : OpticalLayer, 
+                wavefront       : Wavefront,
+                figsize         : tuple = (10, 4),
+                dpi             : int = 120,
+                angular_units   : str = 'radians', 
+                cartesian_units : str = 'meters', 
+                sigfigs         : int = 4) -> None:
+        """
+        Displays a plot of the wavefront amplitude and opd or phase.
+
+        Parameters
+        ----------
+        wavefront : Wavefront
+            The dummy wavefront to propagate though the optics.
+        figsize : tuple = (10, 4)
+            The size of the figure to display.
+        cmap : str = 'inferno'
+            The colour map to use.
+        dpi : int = 120
+            The resolution of the figure.
+        angular_units : str = 'radians'
+            The angular units to use in the summary. Options are 'radians', 
+            'degrees', 'arcseconds' and 'arcminutes'.
+        cartesian_units : str = 'meters'
+            The cartesian units to use in the summary. Options are 'meters',
+            'millimeters' and 'microns'.
+        sigfigs : int = 4
+            The number of significant figures to use in the summary.
+        """
+        amplitude = wavefront.amplitude
+        phase = convert_angular(wavefront.phase, "radians", angular_units)
+        two_image_plot(amplitude[0], phase[0], figsize=figsize, 
+            titles=("Amplitude", "Phase"), cbar_labels=("Intensity", 
+            f"Phase ({angular_units})"), cmaps=('inferno', 'twilight'), 
+            bounds=(None, 2*np.pi), dpi=dpi)
 
 
 class CreateWavefront(OpticalLayer):
@@ -125,7 +187,7 @@ class CreateWavefront(OpticalLayer):
     """
     npixels        : int
     diameter       : Array
-    wavefront_type : str = static_field()
+    wavefront_type : str
 
 
     def __init__(self           : OpticalLayer,
@@ -224,6 +286,33 @@ class CreateWavefront(OpticalLayer):
             return wavefront, parameters
         else:
             return wavefront
+    
+
+    def summary(self            : OpticalLayer, 
+                angular_units   : str = 'radians', 
+                cartesian_units : str = 'meters', 
+                sigfigs         : int = 4) -> str:
+        """
+        Returns a summary of the class.
+
+        Parameters
+        ----------
+        angular_units : str = 'radians'
+            The angular units to use in the summary. Options are 'radians', 
+            'degrees', 'arcseconds' and 'arcminutes'.
+        cartesian_units : str = 'meters'
+            The cartesian units to use in the summary. Options are 'meters',
+            'millimeters' and 'microns'.
+        sigfigs : int = 4
+            The number of significant figures to use in the summary.
+
+        Returns
+        -------
+        summary : str
+            A summary of the class.
+        """
+        return (f"{self.name}: Creates a {self.wavefront_type} wavefront of " 
+                f"size {self.npixels} pixels and diameter {self.diameter} m.")
 
 
 class TiltWavefront(OpticalLayer):
@@ -279,6 +368,34 @@ class TiltWavefront(OpticalLayer):
         return wavefront.tilt_wavefront(self.tilt_angles)
 
 
+    def summary(self            : OpticalLayer, 
+                angular_units   : str = 'radians', 
+                cartesian_units : str = 'meters', 
+                sigfigs         : int = 4) -> str:
+        """
+        Returns a summary of the class.
+
+        Parameters
+        ----------
+        angular_units : str = 'radians'
+            The angular units to use in the summary. Options are 'radians', 
+            'degrees', 'arcseconds' and 'arcminutes'.
+        cartesian_units : str = 'meters'
+            The cartesian units to use in the summary. Options are 'meters',
+            'millimeters' and 'microns'.
+        sigfigs : int = 4
+            The number of significant figures to use in the summary.
+
+        Returns
+        -------
+        summary : str
+            A summary of the class.
+        """
+        angles = convert_angular(self.tilt_angles, 'radians', angular_units)
+        return (f"{self.name}: Tilts the wavefront by {angles:.{sigfigs}} "
+                f"{angular_units} in the (x, y) dimension.")
+
+
 class NormaliseWavefront(OpticalLayer):
     """
     Normalises the input wavefront using the in-built wavefront normalisation
@@ -289,7 +406,7 @@ class NormaliseWavefront(OpticalLayer):
     name : str
         The name of the layer, which is used to index the layers dictionary.
     """
-
+    
 
     def __init__(self : OpticalLayer,
                  name : str = 'NormaliseWavefront') -> OpticalLayer:
@@ -320,6 +437,32 @@ class NormaliseWavefront(OpticalLayer):
             The wavefront with the wavefront normalisation method applied.
         """
         return wavefront.normalise()
+    
+
+    def summary(self            : OpticalLayer, 
+                angular_units   : str = 'radians', 
+                cartesian_units : str = 'meters', 
+                sigfigs         : int = 4) -> str:
+        """
+        Returns a summary of the class.
+
+        Parameters
+        ----------
+        angular_units : str = 'radians'
+            The angular units to use in the summary. Options are 'radians', 
+            'degrees', 'arcseconds' and 'arcminutes'.
+        cartesian_units : str = 'meters'
+            The cartesian units to use in the summary. Options are 'meters',
+            'millimeters' and 'microns'.
+        sigfigs : int = 4
+            The number of significant figures to use in the summary.
+
+        Returns
+        -------
+        summary : str
+            A summary of the class.
+        """
+        return f"{self.name}: Normalises the wavefront to unity power."
 
 
 class ApplyBasisOPD(OpticalLayer):
@@ -377,7 +520,7 @@ class ApplyBasisOPD(OpticalLayer):
         "First dimension of the basis array.")
 
 
-    def get_total_opd(self : OpticalLayer) -> Array:
+    def get_opd(self : OpticalLayer) -> Array:
         """
         A function to calculate the total OPD from the basis vector and the
         coefficients.
@@ -405,7 +548,34 @@ class ApplyBasisOPD(OpticalLayer):
         wavefront : Wavefront
             The wavefront with the appropriate phase applied.
         """
-        return wavefront.add_opd(self.get_total_opd())
+        return wavefront.add_opd(self.get_opd())
+    
+
+    def summary(self            : OpticalLayer, 
+                angular_units   : str = 'radians', 
+                cartesian_units : str = 'meters', 
+                sigfigs         : int = 4) -> str:
+        """
+        Returns a summary of the class.
+
+        Parameters
+        ----------
+        angular_units : str = 'radians'
+            The angular units to use in the summary. Options are 'radians', 
+            'degrees', 'arcseconds' and 'arcminutes'.
+        cartesian_units : str = 'meters'
+            The cartesian units to use in the summary. Options are 'meters',
+            'millimeters' and 'microns'.
+        sigfigs : int = 4
+            The number of significant figures to use in the summary.
+
+        Returns
+        -------
+        summary : str
+            A summary of the class.
+        """
+        return (f"{self.name}: Applies an Optical Path Difference (OPD) to the "
+            f"wavefront calculated from the basis vectors and coefficients.")
 
 
 class AddPhase(OpticalLayer):
@@ -432,8 +602,8 @@ class AddPhase(OpticalLayer):
         ----------
         phase : Array, radians
             Array of phase values to be applied to the input wavefront. This
-            must a 0, 2 or 3 dimensional array with equal to that of the wavefront
-            at time of aplication.
+            must a 0, 2 or 3 dimensional array with equal to that of the 
+            wavefront at time of aplication.
         name : str = 'AddPhase'
             The name of the layer, which is used to index the layers dictionary.
         """
@@ -460,6 +630,32 @@ class AddPhase(OpticalLayer):
             The wavefront with the phase added.
         """
         return wavefront.add_phase(self.phase)
+    
+
+    def summary(self            : OpticalLayer, 
+                angular_units   : str = 'radians', 
+                cartesian_units : str = 'meters', 
+                sigfigs         : int = 4) -> str:
+        """
+        Returns a summary of the class.
+
+        Parameters
+        ----------
+        angular_units : str = 'radians'
+            The angular units to use in the summary. Options are 'radians', 
+            'degrees', 'arcseconds' and 'arcminutes'.
+        cartesian_units : str = 'meters'
+            The cartesian units to use in the summary. Options are 'meters',
+            'millimeters' and 'microns'.
+        sigfigs : int = 4
+            The number of significant figures to use in the summary.
+
+        Returns
+        -------
+        summary : str
+            A summary of the class.
+        """
+        return f"{self.name}: Add an array of phase values to the wavefront."
 
 
 class AddOPD(OpticalLayer):
@@ -514,6 +710,33 @@ class AddOPD(OpticalLayer):
             The wavefront with the OPD added.
         """
         return wavefront.add_opd(self.opd)
+    
+
+    def summary(self            : OpticalLayer, 
+                angular_units   : str = 'radians', 
+                cartesian_units : str = 'meters', 
+                sigfigs         : int = 4) -> str:
+        """
+        Returns a summary of the class.
+
+        Parameters
+        ----------
+        angular_units : str = 'radians'
+            The angular units to use in the summary. Options are 'radians', 
+            'degrees', 'arcseconds' and 'arcminutes'.
+        cartesian_units : str = 'meters'
+            The cartesian units to use in the summary. Options are 'meters',
+            'millimeters' and 'microns'.
+        sigfigs : int = 4
+            The number of significant figures to use in the summary.
+
+        Returns
+        -------
+        summary : str
+            A summary of the class.
+        """
+        return (f"{self.name}: Add an array of Optical Path Differences (OPD) "
+                "to the wavefront.")
 
 
 class TransmissiveOptic(OpticalLayer):
@@ -571,246 +794,33 @@ class TransmissiveOptic(OpticalLayer):
             The wavefront with the tranmission applied.
         """
         return wavefront.multiply_amplitude(self.transmission)
+    
 
-
-class CompoundAperture(OpticalLayer):
-    """
-    Applies a series of soft-edged, circular aperture and occulters, defined by
-    their physical (x, y) positions and radii. Coordinates are taken from the
-    wavefront.
-
-    Attributes
-    ----------
-    aperture_radii : Array, meters
-        The array of radii of the apertures.
-    aperture_coords : Array, meters
-        The array of (x, y) coordinates of the centers of the apertures.
-    occulter_radii : Array, meters
-        The array of radii of the occulters.
-    occulter_coords : Array, meters
-        The array of (x, y) coordinates of the centers of the occulters.
-    name : str
-        The name of the layer, which is used to index the layers dictionary.
-    """
-    aperture_radii  : Array
-    aperture_coords : Array
-    occulter_radii  : Array
-    occulter_coords : Array
-
-
-    def __init__(self            : OpticalLayer,
-                 aperture_radii  : Array,
-                 aperture_coords : Array = None,
-                 occulter_radii  : Array = None,
-                 occulter_coords : Array = None,
-                 name            : str   = 'CompoundAperture') -> OpticalLayer:
+    def summary(self            : OpticalLayer, 
+                angular_units   : str = 'radians', 
+                cartesian_units : str = 'meters', 
+                sigfigs         : int = 4) -> str:
         """
-        Constructor for the CompoundAperture class.
+        Returns a summary of the class.
 
         Parameters
         ----------
-        aperture_radii : Array, meters
-            The array of radii of the apertures.
-        aperture_coords : Array, meters
-            The array of (x, y) coordinates of the centers of the apertures.
-        occulter_radii : Array, meters
-            The array of radii of the occulters.
-        occulter_coords : Array, meters
-            The array of (x, y) coordinates of the centers of the occulters.
-        name : str = 'CompoundAperture'.
-            The name of the layer, which is used to index the layers dictionary.
-        """
-        super().__init__(name)
-
-        # Get aperture radii and propmote to at least 1d
-        self.aperture_radii = np.asarray(aperture_radii, dtype=float)
-        self.aperture_radii = np.atleast_1d(self.aperture_radii)
-
-        # Ensure corred dimensionality before using shape
-        assert self.aperture_radii.ndim == 1, \
-        ("aperture_radii must be one dimensional.")
-
-        # Construct an empty occulter radii if none is provided
-        self.occulter_radii = np.array([]) if occulter_radii is None else \
-                              np.asarray(occulter_radii, dtype=float)
-        self.occulter_radii = np.atleast_1d(self.occulter_radii)
-
-        # Ensure corred dimensionality before using shape
-        assert self.occulter_radii.ndim == 1, \
-        ("occulter_radii must be one dimensional.")
-
-        # Construct at center if no coordinates are supplied
-        if aperture_coords is None:
-            self.aperture_coords = np.zeros([self.aperture_radii.shape[0], 2])
-        else:
-            self.aperture_coords = np.asarray(aperture_coords, dtype=float)
-            assert self.aperture_coords.ndim == 2 and \
-            self.aperture_coords.shape[0] == self.aperture_radii.shape[0] and \
-            self.aperture_coords.shape[1] == 2, \
-            ("aperture_coords must a 2d array with leading dimension equal to "
-            "The length of aperture_radii, and second dimension equal to 2, ie "
-            "shape == (napertures, 2) ie [(x0, y0), (x1, y1) ...]")
-
-        # Construct at center if no coordinates are supplied
-        if occulter_coords is None:
-            self.occulter_coords = np.zeros([self.occulter_radii.shape[0], 2])
-        else:
-            self.occulter_coords = np.asarray(occulter_coords, dtype=float)
-            assert self.occulter_coords.ndim == 2 and \
-            self.occulter_coords.shape[0] == self.occulter_radii.shape[0] and \
-            self.occulter_coords.shape[1] == 2, \
-            ("occulter_coords must a 2d array with leading dimension equal to "
-            "The length of occulter_radii, and second dimension equal to 2, ie "
-            "shape == (nocculters, 2) ie [(x0, y0), (x1, y1) ...]")
-
-
-    def make_aperture(self        : OpticalLayer,
-                      radius      : Array,
-                      center      : Array,
-                      xycoords    : Array,
-                      is_aperture : Array,
-                      vmin        : float = 0.,
-                      vmax        : float = 1.) -> Array:
-        """
-        Constructs a soft-edged aperture or occulter.
-
-        Parameters
-        ----------
-        radius : Array, meters
-            The radius of the aperture/occulter.
-        center : Array, meters
-            The (x, y) center of the aperture/occulter.
-        xycoords : Array, meters
-            The (xcoordinates, ycoordinates) arrays to calculate the
-            apertures/occulters upon.
-        is_aperture : bool
-            Determines whether an aperture (True) or occulter (False) is
-            calculated.
-        vmin : float = 0.
-            The minimum value to set the transmission to, default is 1e-8 in
-            order to keep gradients stable.
-        vmax : float = 1.
-            The maximum value to set the transmission to, default is 1.
+        angular_units : str = 'radians'
+            The angular units to use in the summary. Options are 'radians', 
+            'degrees', 'arcseconds' and 'arcminutes'.
+        cartesian_units : str = 'meters'
+            The cartesian units to use in the summary. Options are 'meters',
+            'millimeters' and 'microns'.
+        sigfigs : int = 4
+            The number of significant figures to use in the summary.
 
         Returns
         -------
-        aperture : Array
-            The corresponding aperture or occuler, depending on the is_aperture
-            parameter.
+        summary : str
+            A summary of the class.
         """
-        # Shift coordinates
-        xycoords -= center.reshape(2, 1, 1)
-        rcoords = np.hypot(xycoords[0], xycoords[1])
-        thetacoords = np.arctan2(xycoords[1], xycoords[0])
-
-        # Wrap theta values around circle
-        thetas_mapped = (thetacoords + np.pi/4)%(np.pi/2) - np.pi/4
-
-        # Calculate projected pixel size
-        npixels = xycoords.shape[-1]
-        pixel_scale = (np.max(xycoords) - np.min(xycoords))/(npixels-1)
-        angle = (pixel_scale/2)*np.hypot(1, np.tan(thetas_mapped))
-
-        # Get projected radial distance
-        distance = radius - rcoords
-        if not is_aperture:
-            distance *= -1
-
-        # Fit linear slop slong projected pixel sizes/radial distances
-        m = (vmax-vmin)/(2*angle)
-        b = (vmax-vmin)/2
-        grey = m * distance + b
-
-        # Clip to desired range
-        aperture = np.clip(grey, a_min=vmin, a_max=vmax)
-        return aperture
-
-
-    def construct_combined_aperture(self     : OpticalLayer,
-                                    diameter : Array,
-                                    npixels  : int) -> Array:
-        """
-        Constructs the various apertures and occulters from the stored
-        parameters and combines them into a single transmission array.
-
-        Parameters
-        ----------
-        diameter : Array, meters
-            The diameter of the wavefront to calculate the aperture on.
-        npixels : int
-            The linear size of the array to calculate the aperture on.
-
-        Returns
-        -------
-        aperture : Array
-            The final combined aperture.
-        """
-        # Map aperture function
-        mapped_aperture = vmap(self.make_aperture,
-                                   in_axes=(0, 0, None, None))
-
-        # Generate coordinate grid
-        pixel_scale = diameter/npixels
-        xycoords = dLux.utils.get_pixel_coordinates(npixels, pixel_scale)
-
-        # Generate aperture/occulters
-        outer_apers = mapped_aperture(self.aperture_radii, \
-                                      self.aperture_coords, xycoords, True)
-        inner_apers = mapped_aperture(self.occulter_radii, \
-                                      self.occulter_coords, xycoords, False)
-
-        # Bound values
-        outer_comb = np.clip(outer_apers.sum(0), a_min=0., a_max=1.)
-        inner_comb = np.prod(inner_apers, axis=0)
-
-        # Combine
-        return outer_comb * inner_comb
-
-
-    def get_aperture(self     : OpticalLayer,
-                     diameter : Array = None,
-                     npixels  : int   = 512) -> Array:
-        """
-        Constructs the various apertures and occulters from the stored
-        parameters and combines them into a single transmission array. By
-        defualt this method takes the largest aperture and uses that as the
-        diameters, and defaults to 512 pixels.
-
-        Parameters
-        ----------
-        diameter : Array, meters = None
-            The diameter of the wavefront to calculate the aperture on. Uses
-            the largest aperture by default
-        npixels : int = 512
-            The linear size of the array to calculate the aperture on.
-
-        Returns
-        -------
-        aperture : Array
-            The final combined aperture.
-        """
-        diameter = 2*self.aperture_radii.max() if diameter is None else diameter
-        return self.construct_combined_aperture(diameter, npixels)
-
-
-    def __call__(self : OpticalLayer, wavefront : Wavefront) -> Wavefront:
-        """
-        Generates and applies the combined apertures transmission array to the
-        wavefront.
-
-        Parameters
-        ----------
-        wavefront : Wavefront
-            The wavefront to operate on.
-
-        Returns
-        -------
-        wavefront : Wavefront
-            The wavefront with the combined aperture applied.
-        """
-        aper = self.construct_combined_aperture(wavefront.diameter,
-                                                wavefront.npixels)
-        return wavefront.multiply_amplitude(aper)
+        return (f"{self.name}: Applies an array of tranmission values to the "
+                "Wavefront.")
 
 
 class ApplyBasisCLIMB(OpticalLayer):
@@ -934,7 +944,8 @@ class ApplyBasisCLIMB(OpticalLayer):
 
 
     def lsq_params(self, img):
-        xx, yy = np.meshgrid(np.linspace(0,1,img.shape[0]),np.linspace(0,1,img.shape[1]))
+        xx, yy = np.meshgrid(np.linspace(0,1,img.shape[0]),
+                             np.linspace(0,1,img.shape[1]))
         A = np.vstack([xx.ravel(), yy.ravel(), np.ones_like(xx).ravel()]).T
         matrix = np.linalg.inv(np.dot(A.T,A)).dot(A.T)
         return matrix, xx, yy, A
@@ -947,7 +958,9 @@ class ApplyBasisCLIMB(OpticalLayer):
 
     def area(self, img, epsilon = 1e-15):
         a,b,c = self.lsq(img)
-        a, b, c = np.where(a==0,epsilon,a), np.where(b==0,epsilon,b), np.where(c==0,epsilon,c)
+        a = np.where(a==0,epsilon,a)
+        b = np.where(b==0,epsilon,b)
+        c = np.where(c==0,epsilon,c)
         x1 = (-b-c)/(a) # don't divide by zero
         x2 = -c/(a) # don't divide by zero
         x1, x2 = np.min(np.array([x1,x2])), np.max(np.array([x1,x2]))
@@ -986,6 +999,33 @@ class ApplyBasisCLIMB(OpticalLayer):
         soft_bin = vmap_mask(flat).reshape(ppsz, ppsz)
 
         return soft_bin
+    
+
+    def summary(self            : OpticalLayer, 
+                angular_units   : str = 'radians', 
+                cartesian_units : str = 'meters', 
+                sigfigs         : int = 4) -> str:
+        """
+        Returns a summary of the class.
+
+        Parameters
+        ----------
+        angular_units : str = 'radians'
+            The angular units to use in the summary. Options are 'radians', 
+            'degrees', 'arcseconds' and 'arcminutes'.
+        cartesian_units : str = 'meters'
+            The cartesian units to use in the summary. Options are 'meters',
+            'millimeters' and 'microns'.
+        sigfigs : int = 4
+            The number of significant figures to use in the summary.
+
+        Returns
+        -------
+        summary : str
+            A summary of the class.
+        """
+        return (f"{self.name}: Applies a binary OPD to the Wavefront using the "
+                "CLIMB algorithm.")
 
 
 class Rotate(OpticalLayer):
@@ -1001,6 +1041,9 @@ class Rotate(OpticalLayer):
         or the real and imaginary arrays.
     fourier : bool
         Should the rotation be done using fourier methods or interpolation.
+    order : int = 1
+        The order of the interpolation to use. Only applies if fourier is
+        False. Must be 0, 1, or 3.
     padding : int
         The amount of padding to use if the fourier method is used.
     name : str
@@ -1009,6 +1052,7 @@ class Rotate(OpticalLayer):
     angle          : Array
     real_imaginary : bool
     fourier        : bool
+    order          : int
     padding        : int
 
 
@@ -1016,6 +1060,7 @@ class Rotate(OpticalLayer):
                  angle          : Array,
                  real_imaginary : bool = False,
                  fourier        : bool = False,
+                 order          : int  = 1,
                  padding        : int  = None,
                  name           : str  = 'Rotate') -> OpticalLayer:
         """
@@ -1024,13 +1069,17 @@ class Rotate(OpticalLayer):
         Parameters
         ----------
         angle: float, radians
-            The angle by which to rotate the wavefront in the clockwise direction.
+            The angle by which to rotate the wavefront in the clockwise 
+            direction.
         real_imaginary : bool = False
             Should the rotation be performed on the amplitude and phase array
             or the real and imaginary arrays.
         fourier : bool = False
             Should the fourier rotation method be used (True), or regular
             interpolation method be used (False).
+        order : int = 1
+            The order of the interpolation to use. Only applies if fourier is
+            False. Must be 0, 1, or 3.
         padding : int = None
             The amount of fourier padding to use. Only applies if fourier is
             True.
@@ -1040,6 +1089,9 @@ class Rotate(OpticalLayer):
         super().__init__(name)
         self.angle          = np.asarray(angle, dtype=float)
         self.real_imaginary = bool(real_imaginary)
+        if order not in (0, 1, 3):
+            raise ValueError("Order must be 0, 1, or 3.")
+        self.order = int(order)
         self.fourier        = bool(fourier)
         self.padding = padding if padding is None else int(padding)
         assert self.angle.ndim == 0, ("angle must be scalar array.")
@@ -1059,6 +1111,45 @@ class Rotate(OpticalLayer):
         wavefront : Wavefront
             The rotated wavefront.
         """
-        args = [self.angle, self.real_imaginary, self.fourier]
+        args = [self.angle, self.real_imaginary, self.fourier, self.order]
         args += [self.padding] if self.padding is not None else []
         return wavefront.rotate(*args)
+
+
+    def summary(self            : OpticalLayer, 
+                angular_units   : str = 'radians', 
+                cartesian_units : str = 'meters', 
+                sigfigs         : int = 4) -> str:
+        """
+        Returns a summary of the class.
+
+        Parameters
+        ----------
+        angular_units : str = 'radians'
+            The angular units to use in the summary. Options are 'radians', 
+            'degrees', 'arcseconds' and 'arcminutes'.
+        cartesian_units : str = 'meters'
+            The cartesian units to use in the summary. Options are 'meters',
+            'millimeters' and 'microns'.
+        sigfigs : int = 4
+            The number of significant figures to use in the summary.
+
+        Returns
+        -------
+        summary : str
+            A summary of the class.
+        """
+        angle = convert_angular(self.angle, 'radians', angular_units)
+        
+        if self.fourier:
+            method = f"a Fourier method with padding of {self.padding}"
+        else:
+            method = "an Interpolation method of order 1"
+
+        if self.real_imaginary:
+            wf_type = "real and imaginary arrays"
+        else:
+            wf_type = "amplitude and phase arrays"
+
+        return (f"{self.name}: Applies a {angle:.{sigfigs}} {angular_units} "
+                f"rotation to the wavefront {wf_type} using {method}.")
\ No newline at end of file
